==> entodicton-6.1.0.tgz: package/src/digraph.js <==
class Digraph {
  constructor (edges = []) {
    this.edges = edges
  }

/*
  edges () {
    return this.edges
  }
*/
  acdcs (s, from, to) {
    const todo = [s]
    const seen = new Set([s])
    const acdcs = new Set([])
    while (todo.length > 0) {
      const n = todo.pop()
      this.edges.forEach((e) => {
        if (e[from] == n) {
          acdcs.add(e[to])
          if (!seen.has(e[to])) {
            todo.push(e[to])
            seen.add(e[to])
          }
        }
      })
    }
    return acdcs
  }

  isA(low, high) {
    if (low == high) {
      return true;
    }
    return this.descendants(low).has(high)
  }

  ancestors (s) {
    return this.acdcs(s, 1, 0)
  }

  descendants (s) {
    return this.acdcs(s, 0, 1)
  }

  minima (nodes) {
    const minima = new Set(nodes)
    const descendants = new Set([])
    nodes.forEach((node) => {
      this.descendants(node).forEach((n) => descendants.add(n))
    })
    descendants.forEach((n) => minima.delete(n))
    return minima
  }

  maxima (nodes) {
    const maxima = new Set(nodes)
    const ancestors = new Set([])
    nodes.forEach((node) => {
      this.ancestors(node).forEach((n) => ancestors.add(n))
    })
    ancestors.forEach((n) => maxima.delete(n))
    return maxima
  }

  addList(l) {
    for (let i = 1; i < l.length; ++i) {
      this.edges.push([l[i-1], l[i]]) 
    }
  }

  order (todo) {
    let ordered = []
    while (todo.length > 0) {
      let nodes = this.minima(todo)
      todo = todo.filter( (e) => !nodes.has( e ) )
      for (const node of nodes) {
        ordered.push(node)
      }
      //ordered = ordered.concat([...nodes])
    }
    return ordered
  }
}

module.exports = Digraph

==> entodicton-6.10.3.tgz: package/src/digraph.js <==
class Digraph {
  constructor (edges = []) {
    this.edges = edges
  }

/*
  edges () {
    return this.edges
  }
*/
  acdcs (s, from, to) {
    const todo = [s]
    const seen = new Set([s])
    const acdcs = new Set([])
    while (todo.length > 0) {
      const n = todo.pop()
      this.edges.forEach((e) => {
        if (e[from] == n) {
          acdcs.add(e[to])
          if (!seen.has(e[to])) {
            todo.push(e[to])
            seen.add(e[to])
          }
        }
      })
    }
    return acdcs
  }

  isA(low, high) {
    if (low == high) {
      return true;
    }
    return this.descendants(low).has(high)
  }

  ancestors (s) {
    return this.acdcs(s, 1, 0)
  }

  descendants (s) {
    return this.acdcs(s, 0, 1)
  }

  minima (nodes) {
    let minima = new Set(nodes)
    const descendants = new Set([])
    nodes.forEach((node) => {
      this.descendants(node).forEach((n) => descendants.add(n))
    })
    descendants.forEach((n) => minima.delete(n))
    if (minima.size == 0) {
      // all unrelated
      minima = new Set(nodes)
    }
    return minima
  }

  maxima (nodes) {
    const maxima = new Set(nodes)
    const ancestors = new Set([])
    nodes.forEach((node) => {
      this.ancestors(node).forEach((n) => ancestors.add(n))
    })
    ancestors.forEach((n) => maxima.delete(n))
    return maxima
  }

  addList(l) {
    for (let i = 1; i < l.length; ++i) {
      this.edges.push([l[i-1], l[i]]) 
    }
  }

  order (todo) {
    let ordered = []
    while (todo.length > 0) {
      let nodes = this.minima(todo)
      todo = todo.filter( (e) => !nodes.has( e ) )
      for (const node of nodes) {
        ordered.push(node)
      }
      //ordered = ordered.concat([...nodes])
    }
    return ordered
  }
}

module.exports = Digraph
